<?php

/**
 * @copyright   Copyright (C) 2009 - 2017 Michael Richey. All rights reserved.
 * @license             GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access
defined('_JEXEC') or die;

class plgSystemH2Push extends JPlugin {
    private $_self;
    private $_dom;
    private $_pushItems;
    private $_doc;
    private $_app;
    private $_cachedata;
    private $_cache;
    private $_omit = array();
    private $_mediaversion;
    private $_init = false;
    public function onAfterRender() {
        $this->_app = JFactory::getApplication();
	$this->_doc = JFactory::getDocument();
	if(!$this->_app->isSite() || $this->_doc->getType() !== 'html' || JBrowser::getHTTPProtocol() != '2.0') {
	    return true;
	}
        $this->_setup();
        if($this->params->get('javascripts',0)) {
            $this->_pushScripts();
        }
        if($this->params->get('stylesheets',0)) {
            $this->_pushStyles();
        }
        if($this->params->get('images',0)) {
            $this->_pushImages();
        }
        $this->_preConnect();
	$this->_pushHeader();
    }
    private function _setup() {
        if(!$this->_init) {            
            $root = parse_url(JURI::root(false));
            $this->_self = $root['host'];
            $this->_push = array();
            $this->_cache = $this->_readCacheItems();  
            $this->_dom = $this->_getDOM();
            $this->_mediaversion = $this->_doc->getMediaVersion();
            $this->_forceItems();
            $this->_init = true;
        }
    }
    private function _push($type,$uri) {
        if(in_array($uri,$this->_omit)) {
            return;
        }
        if(!isset($this->_pushItems[$type])) {
            $this->_pushItems[$type] = array();
        }
        $this->_pushItems[$type][$uri] = $uri;
    }
    private function _forceItems() {
        $forced = $this->params->get('forced',array(),'ARRAY');
        foreach((array)$forced as $item) {
            switch($item->type) {
                case 'omit':
                    $this->_omit[] = $item->path;
                    break;
                default:
                    $uri = $item->path.((bool)$item->version?'?'.$this->_mediaversion:'');
                    $this->_push($item->type,$uri);
                    break;
            }
        }
    }
    private function _getDOM() {             
	$dom = new DOMDocument();
	$source = $this->_app->getBody();	
        $erlevel = error_reporting(0);
        $xhtml = (preg_match('/XHTML/', $source)) ? true : false;
        switch ($xhtml) {
            case true:
                $dom->loadXML($source);
                break;
            case false:
                $dom->loadHTML($source);
                break;
        }
	error_reporting($erlevel);
        return $dom;
    }
    private function _readCacheItems() {
        $data = $this->_app->getUserState('plg_system_h2push',null);
        $this->_cachedata = ($data === null)?array():json_decode($data);
        return $this->_cachedata;
    }
    private function _pushScripts() {
	foreach($this->_doc->_scripts as $uri=>$options) {
            if(isset($options['options']['version']) && $options['options']['version'] === 'auto') {
                $uri.='?'.$this->_mediaversion;
            }
            $pushuri = $this->_examine($uri);
	    $this->_push('script',$pushuri);
	}
        $scripts = @$this->_dom->getElementsByTagName('script');
        error_log(print_r($scripts,true));
        if(count($scripts)) {
            foreach($scripts as $script) {
                $pushuri = $this->_examine($script->attributes->getNamedItem('src')->nodeValue);
                $this->_push('script',$pushuri);
            }
        }
    }
    private function _pushStyles() {
	foreach($this->_doc->_styleSheets as $uri=>$options) {
            if(isset($options['options']['version']) && $options['options']['version'] === 'auto') {
                $uri.='?'.$this->_mediaversion;
            }
            $pushuri = $this->_examine($uri);
	    $this->_push('style',$pushuri);
	}	
    }
    private function _pushImages() {
        $images = @$this->_dom->getElementsByTagName('img');
        if(count($images)) {
            foreach ($images as $img) {
                $pushuri = $this->_examine($img->attributes->getNamedItem('src')->nodeValue);
                $this->_push('image',$pushuri);
            }
        }
    }
    private function _preConnect() {
        $preconnects = $this->params->get('preconnect',array(),'ARRAY');
        if(count($preconnects)) {
            foreach($preconnects as $item) {
                $this->_push('preconnect',$item->host);
            }
        }
    }
    private function _pushHeader() {
	foreach($this->_pushItems as $type=>$a) {
	    foreach($a as $uri) {
                if($uri !== false && strlen(trim($uri)) > 0 && !in_array($uri,$this->_cache)) {
                    $this->_cache[] = $uri;
                    switch($type) {
                        case 'preconnect':
                            $push[] = '<'.$uri.'>; rel=preconnect; crossorigin';
                            break;
                        default:
                            $push[] = '<'.$uri.'>; as='.$type.'; rel=preload';
                            break;
                    }
                }
	    }
	}
	if(count($push)) {
	    $this->_app->setHeader('Link',implode(', ',$push));
	}
        $diff = count(array_diff($this->_cache,$this->_cachedata));
        if($diff > 0) {
            $this->_app->setUserState('plg_system_h2push',json_encode($this->_cache));
        }
    }
    private function _examine($uri){
        $root = parse_url($uri);
        $r = false;
	switch(substr($uri,0,2)) {
	    case '//':
                if($root['host'] === $this->_self) {
                    $r = $root['path'];
                }
		break;
	    case 'ht': // direct link - return as-is
                if($root['host'] === $this->_self) {
                    $r = $root['path'];
                }	
		break;
	    default: // it's a path
		$r = $root['path'];
		break;
	}
        return $r?($r.(isset($root['query'])?'?'.$root['query']:'')):$r;
    }
}
